"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../Logger");
const TimerQueue = require("timer-queue");
const ErrorHelper_1 = require("./ErrorHelper");
const ShopifyProductService_1 = require("./ShopifyProductService");
class ShopifyCollectionService {
    constructor(axiosInstance) {
        this.axiosInstance = axiosInstance;
        this.create = (collection) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Creating collection`, collection);
                    const response = yield this.axiosInstance.post(`/custom_collections.json`, { custom_collection: collection });
                    resolve(response.data.custom_collection);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Creates a collect at the end of the collection
         *
         * @param  {ICollect} collect
         * @returns Promise
         */
        this.createCollect = (collect) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Creating collect`, collect);
                    const response = yield this.axiosInstance.post(`/collects.json`, { collect: collect });
                    resolve(response.data.collect);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Creates a collect at a certain position in the collection
         *
         * @param  {collectionId}
         * @param  {productId}
         * @param  {position} Position within the collection
         * @returns Promise
         */
        this.createCollectAtPosition = (collectionId, productId, position) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Creating collect for product ${productId} and collection ${collectionId} at position ${position}`);
                    yield this.axiosInstance.put(`/custom_collections/${collectionId}.json`, { custom_collection: { id: collectionId, collects: [{ product_id: productId, position: position }] } });
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Removes all collects from Shopify.
         *
         * @param  {number} collectionId
         * @param  {number} interval in ms of every delete call to Shopify. Set to 0 or leave it empty to not queue the calls.
         * @returns Promise
         */
        this.removeCollects = (collectionId, interval) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Delete All collects for collection with ID: ${collectionId} and using interval of ${interval} between API calls`);
                    const collects = yield this.getCollects(collectionId);
                    if (interval && interval > 0) {
                        const collectsQueue = new TimerQueue({ interval: interval, timeout: 10000, retry: 3, retryInterval: 500, autostart: false });
                        for (const collect of collects) {
                            collectsQueue.push(() => __awaiter(this, void 0, void 0, function* () {
                                Logger_1.Logger.info(`Removing collect ${collect.id} for collection ${collectionId}`);
                                yield this.axiosInstance.delete(`/collects/${collect.id}.json`);
                            }));
                        }
                        collectsQueue.on('end', () => {
                            Logger_1.Logger.info('End Collects Remove Queue');
                            resolve();
                        });
                        collectsQueue.on('error', () => {
                            Logger_1.Logger.info('End Collects Queue with errors');
                            resolve();
                        });
                        collectsQueue.start();
                    }
                    else {
                        for (const collect of collects) {
                            Logger_1.Logger.info(`Removing collect ${collect.id} for collection ${collectionId} without Queue`);
                            yield this.axiosInstance.delete(`/collects/${collect.id}.json`);
                        }
                        resolve();
                    }
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Removes a collect from a Shopify collection.
         *
         * @param  {number} collectId
         * @returns Promise
         */
        this.removeCollect = (collectId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Delete a collect for collection with ID: ${collectId}`);
                    yield this.axiosInstance.delete(`/collects/${collectId}.json`);
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getCollects = (collectionId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Getting collects for collection`, collectionId);
                    let collects = [];
                    const countResponse = yield this.axiosInstance.get(`/collects/count.json?collection_id=${collectionId}`);
                    const count = countResponse.data.count;
                    Logger_1.Logger.info(`Counted collects ${count}`);
                    while (collects.length < count) {
                        let getCollectsUrl = `/collects.json?collection_id=${collectionId}&limit=250`;
                        if (collects.length > 1) {
                            getCollectsUrl += `&since_id=${collects[collects.length - 1].id}`;
                        }
                        else {
                            getCollectsUrl += '&since_id=0';
                        }
                        Logger_1.Logger.info(`Get collects URL ${getCollectsUrl}`);
                        const collectsRetrieved = yield this.axiosInstance.get(getCollectsUrl);
                        collects = collects.concat(collectsRetrieved.data.collects);
                    }
                    resolve(collects);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.get = (num) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getting customers`);
                    let products = [];
                    const response = yield this.axiosInstance.get(`/products/count.json`);
                    const count = response.data.count > num ? num : response.data.count;
                    while (products.length < count) {
                        let productsGetURL = '/products.json?limit=250&since_id=';
                        if (products.length > 1) {
                            productsGetURL += `${products[products.length - 1].id}`;
                        }
                        else {
                            productsGetURL += '0';
                        }
                        const productsRetrieved = yield this.axiosInstance.get(productsGetURL);
                        products = products.concat(productsRetrieved.data.orders);
                    }
                    resolve(products.slice(0, count));
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getAll = () => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getting custom collections`);
                    const response = yield this.axiosInstance.get(`/custom_collections.json?limit=250`);
                    resolve(response.data.custom_collections);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.delete = (id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`deleting custom collections ${id}`);
                    yield this.axiosInstance.delete(`/custom_collections/${id}.json`);
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.deleteProductsWithTag = (collectionId, productTag) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const collects = yield this.getCollects(collectionId);
                    collects.forEach((collect) => __awaiter(this, void 0, void 0, function* () {
                        const product = yield this.ShopifyProductService.getById(collect.product_id);
                        let deleteProduct = false;
                        if (product.tags) {
                            Logger_1.Logger.info(`Product ID: ${product.id} has tags: ${product.tags}`);
                            product.tags.split(',').forEach((tag) => {
                                if (tag.trim() === productTag) {
                                    deleteProduct = true;
                                }
                            });
                        }
                        if (deleteProduct) {
                            Logger_1.Logger.info(`Product ID: ${product.id} has tag ${productTag} so we should delete it`);
                            yield this.ShopifyProductService.delete(collect.product_id);
                        }
                    }));
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.update = (collection) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Updating custom collections ${collection.id}`);
                    yield this.axiosInstance.put(`/custom_collections/${collection.id}.json`, { custom_collection: collection });
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.publish = (id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Publish custom collections ${id}`);
                    yield this.update({ id: id, published: true });
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.unpublish = (id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`UnPublish custom collections ${id}`);
                    yield this.update({ id: id, published: false });
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.ShopifyProductService = new ShopifyProductService_1.ShopifyProductService(axiosInstance);
    }
}
exports.ShopifyCollectionService = ShopifyCollectionService;
//# sourceMappingURL=ShopifyCollectionService.js.map