import { AxiosInstance } from 'axios';
export declare class ShopifyCustomerService {
    private axiosInstance;
    constructor(axiosInstance: AxiosInstance);
    getById: (customerId: number) => Promise<ICustomer>;
    getByEmail: (email: string) => Promise<ICustomer[]>;
    /**
     *
     * Searches using /customers/search.json?query= and returns the customers that has the tag doing a filter on the endpoint results.
     *
     * @param  {string} tag to search for.
     * @returns Promise
     */
    getByTag: (tag: string) => Promise<ICustomer[]>;
    /**
     *
     * Searches using /customers/search.json?query= and returns the customers that contains the string in the last name field.
     *
     * @param  {string} text to search for.
     * @returns Promise
     */
    getByLastNameContains: (text: string) => Promise<ICustomer[]>;
    create: (customer: ICustomer) => Promise<ICustomer>;
    update: (custormerId: number, customer: ICustomer) => Promise<ICustomer>;
    get: (num: number) => Promise<ICustomer[]>;
    /**
     * Adds an address to a customer.
     *
     * @param  {number} customerId - Customer Id
     * @param  {IAddress} address
     * @returns Promise
     */
    addAddress: (customerId: number, address: IAddress) => Promise<IAddress>;
    /**
     * Adds an address to a customer as its default.
     *
     * @param  {number} customerId - Cusotmer Id
     * @param  {IAddress} address
     * @returns Promise
     */
    addAddressAsDefault: (customerId: number, address: IAddress) => Promise<IAddress>;
    /**
     * Replaces the default address of a customer for the address provided.
     *
     * @param  {number} customerId - Customer Id
     * @param  {IAddress} address
     * @returns Promise
     */
    replacesAddressAsDefault: (customerId: number, address: IAddress) => Promise<IAddress>;
    /**
     * Converts an address of a customer to its default.
     *
     * @param  {number} customerId - Customer Id
     * @param  {IAddress} addressId
     * @returns Promise
     */
    addressAsDefault: (customerId: number, addressId: number) => Promise<IAddress>;
    /**
     * Adds an address to a cusotmer.
     *
     * @param  {number} id - Cusotmer Id
     * @param  {IAddress} address with id field in
     * @returns Promise
     */
    updateAddress: (id: number, address: IAddress) => Promise<IAddress>;
    /**
     * Removes an address to a customer.
     *
     * @param  {number} customerId - Customer Id
     * @param  {number} addressId
     * @returns Promise
     */
    removeAddress: (customerId: number, addressId: number) => Promise<void>;
    /**
     * Gets all metafields for a customer or empty array.
     *
     * @param  {number} id - Cusotmer Id
     * @returns Promise
     */
    getMetafields: (id: number) => Promise<IMetafield[]>;
    /**
     * Gets one concrete metafield for a customer or null if it does not exists.
     *
     * @param  {number} id - Cusotmer Id
     * @param  {string} namespace
     * @param  {string} key
     * @returns Promise
     */
    getMetafield: (id: number, namespace: string, key: string) => Promise<IMetafield>;
    /**
     * Sends invite to a customer.
     *
     * @param  {number} customerId
     * @returns Promise
     */
    sendInvite: (customerId: number) => Promise<void>;
    /**
     * Creates a metafield for a customer and return the created metafield.
     *
     * @param  {number} id
     * @param  {IMetafield} metafield
     * @returns Promise
     */
    postMetafield: (id: number, metafield: IMetafield) => Promise<IMetafield>;
    /**
     * Updates a metafield for a customer and return the updated metafield. <br>
     * The functionality first retrieves the customer metafield to get the ID and then updates it.
     *
     * @param  {number} customerId
     * @param  {IMetafield} metafield.
     * @returns Promise
     */
    putMetafield: (customerId: number, metafield: IMetafield) => Promise<IMetafield>;
    /**
     * Deletes a metafield for a customer.
     *
     * @param  {number} id - Customer Id
     * @param  {number} metafieldId
     * @returns Promise
     */
    deleteMetafield: (id: number, metafieldId: number) => Promise<void>;
}
