"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../Logger");
const ErrorHelper_1 = require("./ErrorHelper");
class ShopifyInventoryService {
    constructor(axiosInstance) {
        this.axiosInstance = axiosInstance;
        /**
         *
         * @param  {number} inventoryId
         * @param  {boolean} trackable. Inventory quantity is traqcked by shopify
         * @returns Promise
         */
        this.setItemToBe = (inventoryId, trackable) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`setItemToTrackable ${inventoryId}`);
                    const response = yield this.axiosInstance.put(`/inventory_items/${inventoryId}.json`, { inventory_item: { id: inventoryId, tracked: trackable } });
                    resolve(response.data.inventory_item);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Updates Inventory cost per item.
         *
         * @param  {number} inventoryId
         * @param  {number} cost. New cost per item
         * @returns Promise
         */
        this.setItemCostToBe = (inventoryId, cost) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`setItemCostToBe ${inventoryId}`);
                    const response = yield this.axiosInstance.put(`/inventory_items/${inventoryId}.json`, { inventory_item: { id: inventoryId, cost: cost } });
                    resolve(response.data.inventory_item);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Updates Inventory cost & sku per item.
         *
         * @param  {number} inventoryId
         * @param  {number} cost. New cost per item
         * @param  {string} sku. New sku per item
         * @returns Promise
         */
        this.setItemCostAndSkuToBe = (inventoryId, cost, sku) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`setItemCostAndSkuToBe ${inventoryId}`);
                    const response = yield this.axiosInstance.put(`/inventory_items/${inventoryId}.json`, { inventory_item: { id: inventoryId, cost: cost, sku: sku } });
                    resolve(response.data.inventory_item);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Sets a quantity to the inventory level
         *
         * @param  {number} inventoryId
         * @param  {number} qty Absolute amount of Qty
         * @param  {number} locationId
         * @returns Promise
         */
        this.setQuantity = (inventoryId, qty, locationId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`setQuantity ${qty} in inventory ID ${inventoryId}`);
                    const response = yield this.axiosInstance.post(`/inventory_levels/set.json`, { location_id: locationId, inventory_item_id: inventoryId, available: qty });
                    resolve(response.data.inventory_level);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Adjusts quantity to the inventory level. So it adds or removes the quantity from the actual value
         *
         * @param  {number} inventoryId
         * @param  {number} qty Absolute amount of Qty
         * @param  {number} locationId
         * @returns Promise
         */
        this.adjustQuantity = (inventoryId, qtyToAdjust, locationId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`adjustsQuantity ${qtyToAdjust} in inventory ID ${inventoryId}`);
                    const response = yield this.axiosInstance.post(`/inventory_levels/adjust.json`, { location_id: locationId, inventory_item_id: inventoryId, available_adjustment: qtyToAdjust });
                    resolve(response.data.inventory_level);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Adjusts quantity to the inventory levels of variant. So it adds or removes the quantity from the actual value.
         * - If quantity is lower than the inventory level. It gets removed from it.
         * - If quantity is bigger from the inventory level but there are more than one level it tries to remove a bit of stock from each level.
         * - If quantity is bigger than the invenotry level and there isn't anymore levels then it removes the stock. A negative value will be placed
         *
         * @param  {number} inventoryId
         * @param  {number} qty Absolute amount of Qty
         * @param  {number} locationId
         * @returns Promise
         */
        this.adjustQuantityOfInventoryItem = (inventoryItemId, qtyToAdjust) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`adjustsQuantity for inventoryItem ${inventoryItemId}`);
                    const inventoryLevels = yield this.getLevel(inventoryItemId);
                    Logger_1.Logger.info(`Inventory levels ${JSON.stringify(inventoryLevels)}`);
                    let dynamicQtyToAdjust = parseInt(qtyToAdjust.toString(), 10);
                    for (const level of inventoryLevels) {
                        if (level.available >= qtyToAdjust) {
                            this.setQuantity(level.inventory_item_id, level.available - qtyToAdjust, level.location_id);
                            dynamicQtyToAdjust = 0;
                            break;
                        }
                    }
                    if (dynamicQtyToAdjust > 0) {
                        Logger_1.Logger.info(`Not enough stock in any of the levels. We will remove it from the first level`);
                        let levelIndex = 0;
                        const levelNumber = inventoryLevels.length;
                        for (const level of inventoryLevels) {
                            Logger_1.Logger.info(`Level ${JSON.stringify(level)}`);
                            if (level.available >= dynamicQtyToAdjust) {
                                this.setQuantity(level.inventory_item_id, level.available - dynamicQtyToAdjust, level.location_id);
                                dynamicQtyToAdjust = 0;
                                Logger_1.Logger.info(`Removing all from 1st level. Missing Qty ${dynamicQtyToAdjust} & ${level.location_id}`);
                                break;
                            }
                            else if (level.available < dynamicQtyToAdjust && levelIndex !== levelNumber - 1) {
                                this.setQuantity(level.inventory_item_id, 0, level.location_id);
                                dynamicQtyToAdjust = dynamicQtyToAdjust - level.available;
                                Logger_1.Logger.info(`Removing part of it from ${level.inventory_item_id} level. Missing Qty ${dynamicQtyToAdjust} & ${level.location_id}`);
                            }
                            else if (level.available < dynamicQtyToAdjust && levelIndex === levelNumber - 1) {
                                this.setQuantity(level.inventory_item_id, level.available - dynamicQtyToAdjust, level.location_id);
                                dynamicQtyToAdjust = 0;
                                Logger_1.Logger.info(`Removing the rest of it from ${level.inventory_item_id} level. Missing Qty ${dynamicQtyToAdjust} & ${level.location_id}`);
                                break;
                            }
                            else {
                                Logger_1.Logger.info('Option of level and stock not detected');
                            }
                            levelIndex += 1;
                        }
                    }
                    else {
                        Logger_1.Logger.info(`All stock removed from one Level`);
                    }
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getLevel = (inventoryId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getLevel for inventory ID ${inventoryId}`);
                    const response = yield this.axiosInstance.get(`/inventory_levels.json?inventory_item_ids=${inventoryId}`);
                    resolve(response.data.inventory_levels);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Returns a list of all iventory Items of the product.
         *
         * @param  {IProduct} product
         * @returns Promise
         */
        this.getInventoryItemsOfProduct = (product) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`get Inventory Items for product ID ${product.id}`);
                    if (!product || !product.variants) {
                        Logger_1.Logger.info(`No variants for product ID ${product.id}`);
                        resolve([]);
                    }
                    else {
                        const inventoryItemsWithVariant = [];
                        const inventoryItemIds = product.variants.map(v => v.inventory_item_id).join(',');
                        const response = yield this.axiosInstance.get(`/inventory_items.json?ids=${inventoryItemIds}`);
                        const inventoryItems = response.data.inventory_items;
                        if (inventoryItemIds && inventoryItems.length > 0) {
                            for (const inventoryItem of inventoryItems) {
                                const variant = product.variants.find(v => v.inventory_item_id === inventoryItem.id);
                                const inventoryItemWithVariant = Object.assign(inventoryItem, { variant_price: parseFloat(variant.price), variant_id: variant.id });
                                inventoryItemsWithVariant.push(inventoryItemWithVariant);
                            }
                        }
                        resolve(inventoryItemsWithVariant);
                    }
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
    }
}
exports.ShopifyInventoryService = ShopifyInventoryService;
//# sourceMappingURL=ShopifyInventoryService.js.map